/* -*- mode: js2 - indent-tabs-mode: nil - js2-basic-offset: 4 -*- */
const Lang = imports.lang;
const St = imports.gi.St;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const AppDisplay = imports.ui.appDisplay;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;

const Gettext = imports.gettext.domain('mfv-quickoptions');
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

let text, button;

//--------------------------------------------------------------
//	Tweak Parameters
//--------------------------------------------------------------

// Max items on each folder, -1 for unlimited
const MaxItemsMfv = -1;

// Central folders location
let centralFolder = GLib.get_home_dir() + '/.centralFolder/';

//--------------------------------------------------------------
//	FolderMenuItem class
//--------------------------------------------------------------
const FolderMenuItem = new Lang.Class({
	Name: 'FolderMenuItem',
	Extends: PopupMenu.PopupMenuItem,

	_init: function(name, iconName, iconSize, desktopFile) {
		this.parent(name, null);

		this.item = new PopupMenu.PopupMenuItem(name, { activate: true });

		this.deskFile = desktopFile;
		this.keyfile = new GLib.KeyFile();

		if (this.keyfile.load_from_file(this.deskFile, GLib.KeyFileFlags.NONE) == false)
			return

		this.folderName = name;
		this.appName = null;

		let lang = GLib.get_language_names();
		for (let i=0; i<lang.length; i++) {
            try {this.appName = this.keyfile.get_locale_string('Desktop Entry', 'Name', lang[i]);} catch (e) {}
            if (this.appName != null) break;
        }
        if (this.appName == null) {
            try {this.appName = this.keyfile.get_string('Desktop Entry', 'Name');} catch(e) {}
        }

		//this.appName = this.keyfile.get_string('Desktop Entry', 'Name');

        if (iconName != null) {
            if (iconSize == null) iconSize = 16;
            let icon = null;

            // is '/path/to/filename'?
            if (GLib.path_is_absolute(iconName)) {
                let gicon = null;
                try {
                gicon = new Gio.Icon.new_for_string(iconName);
                } catch (e)  {
                global.logError('' + e);
                }
                if (gicon != null) {
                    icon = new St.Icon({icon_size: iconSize});
                    icon.set_gicon(gicon);
                }
            }
            else {
                // is 'name.png', must be 'name'?
                if (iconName[iconName.length-4] == '.') {
                    let ext = iconName.slice(iconName.length-3);
                    if ( (ext == 'png') || (ext == 'svg') || (ext == 'xpm') )
                        iconName = iconName.slice(0, iconName.length-4);
                }
                // is themed name icon
                icon = new St.Icon({ icon_name: iconName, icon_size: iconSize });
            }
            if (icon)
                this.item.addActor(icon, { align: St.Align.END });
        }

		this.connect('activate', Lang.bind(this, this._onConnect));
	}, 

	// Gets the names of all files that the folder associated with this item has
	_getItemsFromFolder: function() {
		let i = 0;
		let retItems = [];

		if(this.folderName != null) {
			let current = Gio.File.new_for_path(centralFolder + this.folderName);
			let infos = current.enumerate_children('standard::name,standard::type,standard::size', 0, null);

			let FileInf = null;
			while ((FileInf = infos.next_file(null))) {
				retItems[i] = FileInf.get_attribute_as_string('standard::name');
				i++
			}
		}	

		return retItems;
	},

	// Click event handler for the menu item
	_onConnect: function() {
		let tfile = Gio.File.new_for_path(this.deskFile);
		items = this._getItemsFromFolder();

		if(items.indexOf(tfile.get_basename()) != -1) {
			Main.notify(_("Add to Central Launcher"), _("The file you're trying to add is already on the folder."));		
		} else if(items.length < MaxItemsMfv || MaxItemsMfv == -1) {
			let sfile = Gio.File.new_for_path(this.deskFile);
			
			let dfile = Gio.File.new_for_path(centralFolder + this.folderName + "/" + sfile.get_basename());
			
			let callback = null;
			sfile.copy(dfile, 0, null, null, callback);
	
			let notifyString = _("$app added to $folder folder")
			notifyString = notifyString.replace("$app", this.appName);
			notifyString = notifyString.replace("$folder", this.folderName);
			Main.notify(_("Add to Central Launcher"), notifyString);
		} else {
			Main.notify(_("Add to Central Launcher"), _("The selected folder is full. Remove an icon from it first."));
		}
	},

	_getItem: function() {
		return this.item;
	},

	_imageMenuItem: function(name, iconName, iconSize) {
        return 0;
    }
});

//--------------------------------------------------------------
//	QuickOptionsMenuExtension class
//--------------------------------------------------------------
const QuickOptionsMenuExtension = new Lang.Class({
	Name: 'QuickOptionsMenuExtension',

	_init: function() {
		this.shellApp = AppDisplay.AppIconMenu;
		this.origAppIconMenu_redisplay = null;
		this.centralFolder = GLib.get_home_dir() + '/.centralFolder/';
		this.origAppIconMenu_redisplay = null;
		this.folders = this._getMfvFolders();
		this.items = [];

		if (!this.shellApp) return;
	},

	// Scans central folder and returns the names of its child folders
	_getMfvFolders: function() {
		let retFolders = [];
		let current = Gio.File.new_for_path(this.centralFolder);
		let infos = current.enumerate_children('standard::name,standard::type,standard::size', 0, null);
		let i = 0;

		let FileInf = null;
		while ((FileInf = infos.next_file(null))) {
			if (FileInf.get_file_type() == Gio.FileType.DIRECTORY) {
				retFolders[i] = utf8_decode(FileInf.get_attribute_as_string('standard::name')); 
				i += 1;
			}
		}

		return retFolders;
	},

	// Updates the folders (in case one is deleted or renamed)
	_refreshFolders: function() {
		this.folders = this._getMfvFolders();
	},

	_createQuickOptions: function(shellApp, popUpMenu) {
		if (!shellApp || !popUpMenu) return;
   		if (shellApp.is_window_backed()) return;

		let app = shellApp.get_app_info();
		this.desktopFile = app.get_filename();

		if (GLib.file_test(this.desktopFile, GLib.FileTest.EXISTS) == false)
			return;

		this.submenu = new PopupMenu.PopupSubMenuMenuItem(_("Add to Central Launcher"));
		popUpMenu.addMenuItem(this.submenu);

		let n = 0;
		if (this.folders != null && this.submenu != null) {
			for (let i = 0; i < this.folders.length; i++) {
				let item = new FolderMenuItem(this.folders[i], null, null, this.desktopFile);
				this.items[i] = item;
				this.submenu.menu.addMenuItem(this.items[i], n);
				n++;
			}
		}
	}
});

let QuickOptions = new QuickOptionsMenuExtension();
function init(metadata) {
	Convenience.initTranslations();	
}

function enable() {
	QuickOptions.origAppIconMenu_redisplay = AppDisplay.AppIconMenu.prototype._redisplay;
	AppDisplay.AppIconMenu.prototype._redisplay = function() {
	    QuickOptions.origAppIconMenu_redisplay.call(this);
		QuickOptions._refreshFolders();
		QuickOptions._createQuickOptions(this._source.app, this);
	};
}
function disable() {
	AppDisplay.AppIconMenu.prototype._redisplay = QuickOptions.origAppIconMenu_redisplay;
}
function utf8_decode (str_data) {
  // From: http://phpjs.org/functions
  // +   original by: Webtoolkit.info (http://www.webtoolkit.info/)
  // +      input by: Aman Gupta
  // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +   improved by: Norman "zEh" Fuchs
  // +   bugfixed by: hitwork
  // +   bugfixed by: Onno Marsman
  // +      input by: Brett Zamir (http://brett-zamir.me)
  // +   bugfixed by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
  // +   bugfixed by: kirilloid
  // *     example 1: utf8_decode('Kevin van Zonneveld');
  // *     returns 1: 'Kevin van Zonneveld'

  var tmp_arr = [],
    i = 0,
    ac = 0,
    c1 = 0,
    c2 = 0,
    c3 = 0,
    c4 = 0;

  str_data += '';

  while (i < str_data.length) {
    c1 = str_data.charCodeAt(i);
    if (c1 == 92) {
      var tmp_hex = String.fromCharCode(parseInt(str_data.substring(i+2,i+4),16));
      tmp_hex += String.fromCharCode(parseInt(str_data.substring(i+6,i+8),16));
      tmp_arr[ac]=utf8_decode(tmp_hex);
      i += 8;
      ac ++;
    } else if (c1 <= 191) {
      tmp_arr[ac++] = String.fromCharCode(c1);
      i++;
    } else if (c1 <= 223) {
      c2 = str_data.charCodeAt(i + 1);
      tmp_arr[ac++] = String.fromCharCode(((c1 & 31) << 6) | (c2 & 63));
      i += 2;
    } else if (c1 <= 239) {
      // http://en.wikipedia.org/wiki/UTF-8#Codepage_layout
      c2 = str_data.charCodeAt(i + 1);
      c3 = str_data.charCodeAt(i + 2);
      tmp_arr[ac++] = String.fromCharCode(((c1 & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
      i += 3;
    } else {
      c2 = str_data.charCodeAt(i + 1);
      c3 = str_data.charCodeAt(i + 2);
      c4 = str_data.charCodeAt(i + 3);
      c1 = ((c1 & 7) << 18) | ((c2 & 63) << 12) | ((c3 & 63) << 6) | (c4 & 63);
      c1 -= 0x10000;
      tmp_arr[ac++] = String.fromCharCode(0xD800 | ((c1>>10) & 0x3FF));
      tmp_arr[ac++] = String.fromCharCode(0xDC00 | (c1 & 0x3FF));
      i += 4;
    }
  }

  return tmp_arr.join('');
}
	
